/*
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.classfile;

/**
 * Constants used in JVM class files.
 */
public interface ClassFileConstants
{
    /**
     * Magic number to identify a .class file.
     */
    int JAVA_MAGIC = 0xCAFEBABE;

    /**
     * The major class file version number generated by this library.
     */
    int JAVA_MAJOR = 45;

    /**
     * The minor class file version number generated by this library.
     */
    int JAVA_MINOR = 3;

    /**
     * Type for an UTF8-string constant pool entry .
     */
    byte CST_UTF8 = 1;

    /**
     * Type for an integer constant pool entry.
     */
    byte CST_INTEGER = 3;

    /**
     * Type for 32-bit float constant pool entry.
     */
    byte CST_FLOAT = 4;

    /**
     * Type for 64-bit long integer constant pool entry.
     */
    byte CST_LONG = 5;

    /**
     * Type for 64-bit double constant pool entry.
     */
    byte CST_DOUBLE = 6;

    byte CST_CLASS = 7;

    byte CST_STRING = 8;

    byte CST_FIELD = 9;

    byte CST_METHOD = 10;

    byte CST_INTERFACEMETHOD = 11;

    byte CST_NAMEANDTYPE = 12;

    // ----------------------------------------------------------------------
    // TYPE
    // ----------------------------------------------------------------------

    byte TYP_VOID = 0;
    byte TYP_ADDRESS = 1;
    byte TYP_DOUBLE = 2;
    byte TYP_FLOAT = 3;
    byte TYP_INT = 4;
    byte TYP_LONG = 5;
    byte TYP_REFERENCE = 6;

    // ----------------------------------------------------------------------
    // ACCESS FLAGS
    // ----------------------------------------------------------------------

    int ACC_PUBLIC = 0x0001;
    int ACC_PRIVATE = 0x0002;
    int ACC_PROTECTED = 0x0004;
    int ACC_STATIC = 0x0008;
    int ACC_FINAL = 0x0010;
    int ACC_SYNCHRONIZED = 0x0020;
    int ACC_VOLATILE = 0x0040;
    int ACC_TRANSIENT = 0x0080;
    int ACC_NATIVE = 0x0100;
    int ACC_INTERFACE = 0x0200;
    int ACC_ABSTRACT = 0x0400;
    int ACC_STRICT = 0x0800;

    /**
     * When used as the access flags of a class or interface, this flag
     * indicates special semantics of invokespecial when accessing an
     * overridden method in the superclass. This constant equals
     * ACC_SYNCHRONIZED but these flags are mutually exclusive in their
     * allowed use.
     */
    int ACC_SUPER = 0x0020;

    // ----------------------------------------------------------------------
    // ATTRIBUTES
    // ----------------------------------------------------------------------

    int ATT_GENERIC = -1;
    int ATT_SOURCEFILE = 1;
    int ATT_CONSTANTVALUE = 2;
    int ATT_CODE = 3;
    int ATT_EXCEPTIONS = 4;
    int ATT_LINENUMBERTABLE = 5;
    int ATT_LOCALVARIABLETABLE = 6;
    int ATT_DEPRECATED = 11;
    int ATT_INNERCLASSES = 21;
    int ATT_SYNTHETIC = 22;

    // ----------------------------------------------------------------------
    // POOLED CONSTANTS (FINAL)
    // ----------------------------------------------------------------------

    int POO_ASCII_CONSTANT = 0;
    int POO_CLASS_CONSTANT = 1;
    int POO_DOUBLE_CONSTANT = 2;
    int POO_FLOAT_CONSTANT = 3;
    int POO_INTEGER_CONSTANT = 4;
    int POO_LONG_CONSTANT = 5;
    int POO_STRING_CONSTANT = 6;
    int POO_NAT_CONSTANT = 7;
    int POO_REF_CONSTANT = 8;

    /**
     * Maximum size of Code attribute of a method.
     */
    int MAX_CODE_PER_METHOD = 65535;
}
