/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.classfile;

import java.io.DataOutput;
import java.io.IOException;

/**
 * Wrap an integer constant reference with this CPE.
 */
public class IntegerConstant
    extends PooledConstant
{
    /**
     * Value of integer constant.
     */
    private int value;

    /**
     *
     * @param value Value for integer constant.
     */
    public IntegerConstant(int value)
    {
        this.value = value;
    }

    /**
     * Returns the associated literal.
     */
    public Object getLiteral()
    {
        return new Integer(value);
    }

    /**
     * hashCode (a fast comparison)
     * CONVENTION: return XXXXXXXXXXXX << 4 + Y
     * with Y = ident of the type of the pooled constant
     */
    public final int hashCode()
    {
        return (value << 4) + POO_INTEGER_CONSTANT;
    }

    /**
     * equals (an exact comparison)
     * ASSERT: this.hashCode == o.hashCode ===> cast
     */
    public final boolean equals(Object o)
    {
        return (o instanceof IntegerConstant) &&
            ((IntegerConstant)o).value == value;
    }

    /**
     * Check location of constant value on constant pool.
     *
     * @param constant
     * The already in pooled constant.
     * ASSERT constant.getClass() == this.getClass()
     */
    public final void resolveConstants(PooledConstant constant)
    {
        setIndex(constant.getIndex());
    }

    /**
     * Insert or check location of constant value on constant pool.
     *
     * @param pool
     * The constant pool for this class.
     */
    public void resolveConstants(ConstantPool pool)
    {
        return;
    }

    /**
     * Write this class into the the file (out) getting data position from
     * the constant pool.
     *
     * @param pool
     * The constant pool that contain all data.
     * @param out
     * The file where to write this object info.
     * @exception IOException
     * An io problem has occured.
     */
    public void write(ConstantPool pool, DataOutput out)
        throws IOException
    {
        out.writeByte(CST_INTEGER);
        out.writeInt(value);
    }
}
