/*
 * Copyright (C) 2001 Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.classfile;

import java.io.InputStream;

/**
 * A place holder for a source file.
 */
public class SourceDescription
{
    /**
     * The name of the source file.
     */
    private final String name;

    /**
     * The input stream that reads the source file.
     */
    private final InputStream input;

    /**
     * The length of the source file.
     */
    private final long length;

    /**
     * Whether the location of the source file is writable for a classfile.
     */
    private final boolean writableLocation;

    /**
     * Constructs a SourceDescription object.
     *
     * @param name
     * The name of the source file.
     * @param length
     * The length of the source file.
     * @param input
     * The input stream that reads the source file.
     */
    public SourceDescription(String name, long length,
                             boolean writableLocation, InputStream input)
    {
        this.name = name;
        this.length = length;
        this.writableLocation = writableLocation;
        this.input = input;
    }

    /**
     * Returns the name of the source file.
     *
     * @return
     * The name of the source file.
     */
    public String getName()
    {
        return name;
    }

    /**
     * Returns the input stream that reads the source file.
     *
     * @return
     * The input stream that reads the source file.
     */
    public InputStream getInput()
    {
        return input;
    }

    /**
     * Returns the length of the source file.
     *
     * @return
     * The length of the source file.
     */
    public long getLength()
    {
        return length;
    }

    /**
     * Returns whether the location of the source file is writable for a
     * classfile.
     *
     * @return
     * Whether the location of the source file is writable for a classfile.
     */
    public boolean isWritableLocation()
    {
        return writableLocation;
    }
}
