/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.compiler;

/**
 * An alternative implementation of StringBuffer that should be faster than
 * the original.
 */
public final class FastStringBuffer
{
    /**
     * The cursor; at which index to add new data to the string buffer.
     */
    private int pos;

    /**
     * An array of chars holding the characters in the string buffer.
     */
    private char[] buf = new char[200];


    public void reset()
    {
        pos = 0;
    }

    public String toString()
    {
        return String.valueOf(buf, 0, pos);
    }

    public void append(String s)
    {
        int length = s.length();
        try {
            s.getChars(0, length, buf, pos);
            pos += length;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            grow(length + 16);
            append(s);
        }
    }

    public void append(char c)
    {
        try {
            buf[pos] = c;
            pos++;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            grow(16);
            buf[pos++] = c;
        }
    }

    private void grow(int size)
    {
        char[]  newValue = new char[buf.length + size];

        System.arraycopy(buf, 0, newValue, 0, buf.length);
        buf = newValue;
    }
}

