/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.compiler;

import at.dms.util.FormattedException;
import at.dms.util.Message;
import at.dms.util.MessageDescription;

/**
 * This class is the root class for all compiler errors with a reference
 * to the source text.
 */
public class PositionedError
    extends FormattedException
{
    /**
     * Where the error occurred.
     */
    private final TokenReference where;

    /**
     * An error with a formatted message as argument.
     *
     * @param where
     * The position in the source code.
     * @param message
     * The formatted message.
     */
    public PositionedError(TokenReference where, Message message)
    {
        super(message);
        this.where = where;
    }

    /**
     * An error with an arbitrary number of parameters.
     *
     * @param where
     * The position in the source code.
     * @param description
     * The message description.
     * @param parameters
     * The array of parameters.
     */
    public PositionedError(TokenReference where,
                           MessageDescription description,
                           Object[] parameters)
    {
        super(description, parameters);
        this.where = where;
    }

    /**
     * An error with two parameters.
     *
     * @param where
     * The position in the source code.
     * @param description
     * The message description.
     * @param parameter1
     * The first parameter.
     * @param parameter2
     * The second parameter.
     */
    public PositionedError(TokenReference where,
                           MessageDescription description,
                           Object parameter1,
                           Object parameter2)
    {
        super(description, parameter1, parameter2);
        this.where = where;
    }

    /**
     * An error with one parameter.
     *
     * @param where
     * The position in the source code.
     * @param description
     * The message description.
     * @param parameter
     * The parameter.
     */
    public PositionedError(TokenReference where,
                           MessageDescription description,
                           Object parameter)
    {
        super(description, parameter);
        this.where = where;
    }

    /**
     * An error without parameters.
     *
     * @param where
     * The position in the source code.
     * @param description
     * The message description.
     */
    public PositionedError(TokenReference where,
                           MessageDescription description)
    {
        super(description);
        this.where = where;
    }

    /**
     * Returns the position in the source code.
     */
    public TokenReference getTokenReference()
    {
        return where;
    }

    /**
     * Returns the string explaining the error.
     */
    public String getMessage()
    {
        return ( where.getFile() + ":" + where.getLine()
                 + ":" + super.getMessage() );
    }
}
