/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.classfile.MethodInfo;
import at.dms.compiler.UnpositionedError;
import at.dms.type.*;

/**
 * This class represents a loaded (already compiled) class method.
 */
public class CBinaryMethod
    extends CMethod
{
    /**
     * Constructs method.
     *
     * @param owner
     * The owner of this method.
     * @param methodInfo
     * A method info from a class file.
     */
    public CBinaryMethod(CBinaryClass owner, MethodInfo methodInfo)
        throws UnpositionedError
    {
        super(owner,
              methodInfo.getModifiers(),
              methodInfo.getName(),
              buildReturnType(methodInfo),
              buildParameters(methodInfo),
              buildExceptions(owner, methodInfo),
              methodInfo.isDeprecated());
    }

    /**
     * Creates return type from the method signature.
     */
    private static Type buildReturnType(MethodInfo method)
        throws UnpositionedError
    {
        Type[] types =
            TypeSignature.parseMethodSignature(method.getSignature());

        return types[types.length - 1];
    }

    /**
     * Creates method parameter types from the method signature.
     */
    private static Type[] buildParameters(MethodInfo method)
        throws UnpositionedError
    {
        Type[] signature =
            TypeSignature.parseMethodSignature(method.getSignature());

        Type[] paramTypes = new Type[signature.length - 1];

        for (int i = 0; i < paramTypes.length; i++) {
            paramTypes[i] = signature[i];
        }

        return paramTypes;
    }

    /**
     * Creates types of declared exceptions.
     */
    private static ClassOrInterfaceType[] buildExceptions(CBinaryClass owner,
                                                          MethodInfo method)
        throws UnpositionedError
    {
        String[] exceptions = method.getExceptions();

        if (exceptions == null) {
            return new ClassOrInterfaceType[0];
        }
        else {
            ClassOrInterfaceType[] types =
                new ClassOrInterfaceType[exceptions.length];

            for (int i = 0; i < exceptions.length; i++) {
                types[i] = ClassOrInterfaceType.get(exceptions[i]);
            }
            return types;
        }
    }
}
