/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.type.*;

/**
 * The context in which an expression is used; it is used to check many of
 * the rules of the JLS, and to resolve ambiguities according to the scoping
 * rules of the JLS.
 */
public class CExpressionContext
    extends CContext
{
    /**
     * Whether the value of the whole expression for which this is the context
     * object is discarded. This is used in ExpressionStatement to indicate
     * that the value of the expression is not used. Not all kinds of
     * expressions may have their values discarded, according to the JLS.
     */
    protected boolean discardValue;

    /**
     * Whether this is the context of an expression that is the left side 
     * of an assignment (an LValue)
     */
    protected boolean leftSide;

    /**
     * Creates a new context for evaluation of expressions.
     *
     * @param parent
     * The parent context.
     */
    public CExpressionContext(CBodyContext parent)
    {
        super(parent);
    }

    /**
     * Returns the parent context.
     */
    public CBodyContext getBodyContext()
    {
        return (CBodyContext)parent;
    }

    /**
     * Set this context as a left side in an assignment, so access to vars
     * may be uninitialized.
     *
     * @param leftSide
     * Set the side of current code flow.
     */
    public void setLeftSide(boolean leftSide)
    {
        this.leftSide = leftSide;
    }

    /**
     * Return true if this context is on a left side of an assignment.
     *
     * @return true if this context is on a left side of an assignment.
     */
    public boolean isLeftSide()
    {
        return leftSide;
    }

    /**
     * Sets the 'discardvalue' property.
     */
    public void setDiscardValue(boolean discardValue)
    {
        this.discardValue = discardValue;
    }

    /**
     * Return true if the result value of this expression is discarded.
     *
     * @return true if the result value of this expression is discarded.
     */
    public boolean isDiscardValue()
    {
        return discardValue;
    }
}
