/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.type.*;

import at.dms.compiler.TokenReference;
import at.dms.util.Message;
import at.dms.util.MessageDescription;

/**
 * This class represents Expression errors in error hierarchy.
 */
public class CExpressionError
    extends CLineError
{
    /**
     * An error with a formatted message as argument.
     *
     * @param where       the position in the source code.
     * @param message     the formatted message.
     */
    public CExpressionError(TokenReference where, Message message)
    {
        super(where, message);
    }

    /**
     * An error with an arbitrary number of parameters.
     *
     * @param where       the position in the source code.
     * @param description the message description.
     * @param parameters  the array of parameters.
     */
    public CExpressionError(TokenReference where,
                            MessageDescription description,
                            Object[] parameters)
    {
        super(where, description, parameters);
    }

    /**
     * An error with two parameters.
     *
     * @param where       the position in the source code.
     * @param description the message description.
     * @param parameter1  the first parameter.
     * @param parameter2  the second parameter.
     */
    public CExpressionError(TokenReference where,
                            MessageDescription description,
                            Object parameter1,
                            Object parameter2)
    {
        super(where, description, parameter1, parameter2);
    }

    /**
     * An error with one parameter.
     *
     * @param where       the position in the source code.
     * @param description the message description.
     * @param parameter   the parameter.
     */
    public CExpressionError(TokenReference where,
                            MessageDescription description, Object parameter)
    {
        super(where, description, parameter);
    }

    /**
     * An error without parameters.
     *
     * @param where       the position in the source code.
     * @param description the message description.
     */
    public CExpressionError(TokenReference where,
                            MessageDescription description)
    {
        super(where, description);
    }
}
