/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import at.dms.classfile.FieldInfo;
import at.dms.type.*;

/**
 * This class encapsulates the fields of a class.
 */
public class CFieldTable
    extends at.dms.util.Utils
{
    /**
     * A map of the names of the fields in a class and the corresponding
     * CField objects.
     */
    private Map hashed;

    /**
     * Constructs a field table.
     */
    public CFieldTable(CField[] array)
    {
        hashed = new HashMap(array.length);
        for (int i = 0; i < array.length; i++) {
            hashed.put(array[i].getIdent(), array[i]);
        }
    }

    /**
     * Returns the field with specified name.
     *
     * @param ident
     * The simple name of the field.
     */
    public CField lookup(String ident)
    {
        return (CField)hashed.get(ident);
    }

    /**
     * Returns the number of fields.
     */
    public int size()
    {
        return hashed.size();
    }

    /**
     * Adds a field.
     */
    public void addField(CSourceField field)
    {
        field.setPosition(hashed.size());
        hashed.put(field.getIdent(), field);
    }

    /**
     * Returns the field infos.
     */
    public FieldInfo[] buildFieldInfo()
    {
        FieldInfo[] fields = new FieldInfo[hashed.size()];
        
        for (Iterator iterator = hashed.values().iterator();
             iterator.hasNext(); ) {
            
            CSourceField field = (CSourceField)iterator.next();
            
            fields[field.getPosition()] = field.genFieldInfo();
        }

        return fields;
    }

    /**
     * Debug method.
     */
    void dumpFields()
    {
        Iterator names = hashed.keySet().iterator();
        System.err.println("Field dump:");
        while (names.hasNext()) {
            System.err.println("- " + names.next());
        }
        System.err.println("------------");
    }
}
