/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.type.*;

import at.dms.compiler.UnpositionedError;

/**
 * Interface for all left-value expression.
 */
public interface CInitializable
{

    /**
     * Returns
     * <code>true</code> if this expression is an LValue at call time.
     *
     * @return
     * <code>true</code> if this expression is an LValue at call time.
     * Note: a final variable is a lvalue until it is initialized.
     */
    boolean isLValue(CExpressionContext context);

    /**
     * Initialize this object.
     *
     * @exception UnpositionedError
     * An error if this object can't actually be assigned. This may happen
     * with final variables.
     */
    void initialize(CBodyContext ctxt) throws UnpositionedError;

    /**
     * Returns
     * <code>true</code> if this expression is a variable that is already
     * initialized.
     *
     * @return
     * <code>true</code> if this expression is a variable that is already
     * initialized.
     */
    boolean isInitialized(CExpressionContext context);

    /**
     * Generates a sequence of bytecodes that must be generated before
     * the right-side expression, to store the right side of the assignment
     * expression in this lvalue.
     * Note: in the case of a compound assignment, some operation may be
     * performed on the right side expression before storing.
     *
     * @param code
     * The code list
     */
    void genStartStoreCode(CodeSequence code);

    /**
     * Generates a sequence of bytecodes that must be generated after
     * the right-side expression, to complete the store operation that stores
     * the right side of the assignment expression in this lvalue.
     * Note: in the case of a compound assignment, some operation may be
     * performed on the right side expression before storing.
     *
     * @param discardValue
     * Whether the value of the assignment expression is discarded. (The value
     * of an assignment expression is the value that has been stored in the
     * lvalue.)
     *
     * @param code
     * The code list
     */
    void genEndStoreCode(CodeSequence code, boolean discardValue);
}

