/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.type.*;

import at.dms.classfile.MethodInfo;
import at.dms.classfile.ConstantPool;
import at.dms.classfile.ClassFileFormatException;
import at.dms.classfile.CodeInfo;

/**
 * Methods: VMS 4.6
 *
 * Each method, and each (class or instance) initialization method is
 * described by this structure.
 */
public class CMethodInfo
    extends MethodInfo
{
    /**
     * The method compiled from source.
     */
    private CSourceMethod method;

    /**
     * Constructs a method entry.
     * NOTE:
     * VMS 4.7.5: There must be exactly one Exceptions attribute in each
     * method_info structure.
     *
     * @param modifiers
     * Access permission to the method.
     * @param name
     * The simple name of the method, or <init> or <clinit>.
     * @param type
     * The method signature.
     * @param exceptions
     * The checked exceptions the method may throw.
     * @param code
     * The virtual machine instructions and auxiliary infos.
     * @param deprecated
     * Is this method deprecated?
     * @param synthetic
     * Is this method synthesized by the compiler?
     */
    public CMethodInfo(short modifiers,
                       String name,
                       String type,
                       String[] exceptions,
                       CSourceMethod method,
                       boolean deprecated,
                       boolean synthetic)
    {
        super(modifiers, name, type, exceptions,
              CodeInfo.DUM_INFO, deprecated, synthetic);
        this.method = method;
    }

    /**
     * Insert or check location of constant value on constant pool.
     *
     * @param cp
     * The constant pool for this class.
     */
    public void resolveConstants(ConstantPool cp)
        throws ClassFileFormatException
    {
        CodeInfo code = method.genBodyCode();
        setCodeInfo(code);
        super.resolveConstants(cp);
    }
}
