/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.type.*;

import at.dms.util.Utils;

/**
 * This class represents a throw <throwable> information during check.
 */
public class CThrowableInfo
    extends Utils
{
    /**
     * Type of exception.
     */
    private ClassOrInterfaceType throwable;

    /**
     * Whether subTypes are possible for this throwable, or whether 
     * the property <code>throwable</ocde> is exactly the type that can be
     * thrown. This is necessary for reachability checking.
     */
    private boolean subTypePossible;

    /**
     * Thing that caused the throw.
     */
    private JPhylum location;

    /**
     * Body context in which the throw was done.
     */
    private CBodyContext context;

    /**
     * Whether the throwable is caught.
     */
    private boolean caught;

    /**
     * Constructs a throwable-information handler.
     *
     * @param throwable
     * The type of exception.
     * @param location
     * The throw statement.
     * @param context
     * The state of control flow at throw time.
     */
    public CThrowableInfo(ClassOrInterfaceType throwable,
                          JPhylum location,
                          CBodyContext context,
                          boolean subTypePossible)
    {
        this.throwable = throwable;
        this.location = location;
        this.context = context;
        this.subTypePossible = subTypePossible;
    }

    /**
     * Returns whether subTypes are possible for this throwable, or whether 
     * the property <code>throwable</ocde> is exactly the type that can be
     * thrown. This is necessary for reachability checking.
     */
    public boolean isSubTypePossible()
    {
        return subTypePossible;
    }

    /**
     * Returns the type of exception.
     */
    public ClassOrInterfaceType getThrowable()
    {
        return throwable;
    }

    /**
     * Return the location of this throwable.
     */
    public JPhylum getLocation()
    {
        return location;
    }

    /**
     * Return the context where this throw occurs.
     */
    public CBodyContext getContext()
    {
        return context;
    }

    /**
     * Sets this throwable to be cached or not.
     */
    public void setCaught(boolean caught)
    {
        this.caught = caught;
    }

    /**
     * Return true if this throwable is caught.
     */
    public boolean isCaught()
    {
        return caught;
    }

    public boolean equals(Object o)
    {
        if (o.getClass() == getClass()) {
            CThrowableInfo other = (CThrowableInfo)o;
            return other.throwable.getCClass() == throwable.getCClass();
        }
        else {
            return false;
        }
    }

    public int hashCode()
    {
        return throwable.getCClass().hashCode();
    }
}
