/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.TokenReference;
import at.dms.type.*;

/**
 * A block within a class that is either static or not, and is used to
 * initialize stuff at class-loading or object-instantiation time.
 *
 * All field initializers and explicit ClassBlocks from the source are combined
 * in two synthethic blocks, one for static initialization and one for instance
 * initialization (in addition to the tasks that the constructor performs)
 */
public class JClassBlock
    extends JBlock
{
    /**
     * ...
     */
    private final boolean isStatic;

    /**
     * Construct a node in the parsing tree.
     *
     * @param where
     * The line of this node in the source code.
     * @param isStatic
     * Whether this block is a static initializer.
     * @param body
     * An array of statements.
     */
    public JClassBlock(TokenReference where,
                       boolean isStatic,
                       JStatement[] body)
    {
        super(where, body, null);

        this.isStatic = isStatic;
    }

    /**
     * @return is this block a static initializer
     */
    public boolean isStaticInitializer()
    {
        return isStatic;
    }
}
