/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.type.*;

/**
 * JLS 14.5: Field Statement.
 *
 * a virtual statement that set a field to be fully declared during checkBody
 * of static initializers.
 */
public class JClassFieldDeclarator
    extends JStatement
{
    /**
     * ...
     */
    private JFieldDeclaration declaration;

    /**
     * Construct a node in the parsing tree.
     *
     * @param where
     * The line of this node in the source code.
     * @param declaration
     * The declarator of this field.
     */
    public JClassFieldDeclarator(TokenReference where,
                                 JFieldDeclaration declaration)
    {
        super(where, null);
        this.declaration = declaration;
    }

    /**
     * Check statement.
     *
     * @param context
     * The actual context of analyse.
     * @exception PositionedError
     * If the check fails.
     */
    public void checkStatement(CBodyContext context)
        throws PositionedError
    {
        declaration.checkStatement(context);

        if (declaration.hasInitializer()) {
            ((CSourceField)declaration.getField())
                .setValue(declaration.getVariable().getValue());
        }
    }

    /**
     * Accepts the specified visitor.
     *
     * @param p
     * The visitor.
     */
    public void accept(KjcVisitor p)
    {
    }

    /**
     * Generates a sequence of bytecodes.
     *
     * @param code
     * The code list.
     */
    public void genCode(CodeSequence code)
    {
        JVariableDefinition variable = declaration.getVariable();
        boolean varIsNotConstant = (variable.getValue() != null
                                    && !variable.getValue().isConstant());
        if (!declaration.getField().isFinal()
            || varIsNotConstant
            || declaration.getType().isReference()) {
            
            if (variable.getValue() != null
                && !(variable.getValue().isConstant()
                     && variable.getValue().getLiteral().isDefault())) {
                
                declaration.genCode(code);
            }
        }
    }
}
