/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.type.*;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a parameter declaration in the syntax tree.
 */
public class JFormalParameter
    extends JLocalVariable
{

    /**
     * Construct a node in the parsing tree
     * This method is directly called by the parser
     * @param where  the line of this node in the source code
     * @param ident  the name of this variable
     * @param initializer the initializer
     */
    public JFormalParameter(TokenReference where,
                            int desc,
                            Type type,
                            String ident,
                            boolean isFinal)
    {
        super(where, isFinal ? ACC_FINAL : 0, desc, type, ident, null);
    }

    /**
     * Second pass (quick), check interface looks good
     * Exceptions are not allowed here, this pass is just a tuning
     * pass in order to create informations about exported elements
     * such as Classes, Interfaces, Methods, Constructors and Fields
     * sub classes must check modifiers and call checkInterface(super)
     * @return true iff sub tree is correct enought to check code
     */
    public Type checkInterface(CContext context)
    {
        try {
            type.resolveType(context);
            return type;
        }
        catch (UnpositionedError cue) {
            cue.printStackTrace();
            context.reportTrouble(cue.addPosition(getTokenReference()));
            return StandardType.OBJECT;
        }
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context the actual context of analyse.
     * @return  a pure java expression including promote node.
     * @exception PositionedError Error caught as soon as possible.
     */
    public void checkBody(CBodyContext context)
        throws PositionedError
    {
        try {
            type.resolveType(context);
        }
        catch (UnpositionedError e) {
            throw e.addPosition(getTokenReference());
        }

        try {
            context.getBlockContext().addVariable(this);
        }
        catch (UnpositionedError e) {
            throw e.addPosition(getTokenReference());
        }
        /* All formal parameters are definately assigned when they are bound.
         */
        context.addVariableInfo(getIndex(), CVariableInfo.initialize());
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitFormalParameters(this, isFinal(), getType(), getIdent());
    }
 
    public static final JFormalParameter[] EMPTY = new JFormalParameter[0];
}
