/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.classfile.ClassRefInstruction;
import at.dms.classfile.Instruction;
import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;
import at.dms.type.*;

/**
 * This class represents a instanceof expression.
 */
public class JInstanceofExpression
    extends JExpression
{
    /**
     * ...
     */
    private JExpression expr;

    /**
     * ...
     */
    private Type dest;

    /**
     * Construct a node in the parsing tree.
     *
     * @param where
     * The line of this node in the source code.
     * @param expr
     * The expression to be casted.
     * @param dest
     * The type to test for.
     */
    public JInstanceofExpression(TokenReference where,
                                 JExpression expr,
                                 Type dest)
    {
        super(where);
        this.expr = expr;
        this.dest = dest;
    }

    /**
     * Compute the type of this expression.
     *
     * @return
     * The type of this expression.
     */
    public Type getType()
    {
        return StandardType.BOOLEAN;
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context
     * The actual context of analyse.
     * @return
     * A pure java expression including promote node.
     * @exception PositionedError
     * If the check fails.
     */
    public JExpression checkExpression(CExpressionContext context)
        throws PositionedError
    {
        expr = expr.checkExpression(context);
        try {
            dest.resolveType(context);
        }
        catch (UnpositionedError e) {
            throw e.addPosition(getTokenReference());
        }
        check(context, expr.getType().isReference(),
              KjcMessages.INSTANCEOF_BADTYPE, expr.getType(), dest);

        check(context, dest.allowsCastingConversion(expr.getType()),
              KjcMessages.INSTANCEOF_BADTYPE,
              expr.getType(), dest);

        if (expr.getType().allowsAssignmentConversion(dest)) {
            context.reportTrouble
                (new CWarning(getTokenReference(),
                              KjcMessages.UNNECESSARY_INSTANCEOF, null));
        }

        return this;
    }

    /**
     * Accepts the specified visitor.
     *
     * @param p
     * The visitor.
     */
    public void accept(KjcVisitor p)
    {
        p.visitInstanceofExpression(this, expr, dest);
    }

    /**
     * Generates a sequence of bytecodes.
     *
     * @param code
     * The code list.
     */
    public void genCode(CodeSequence code)
    {
        code.setLineNumber(getTokenReference().getLine());

        expr.genCode(code);

        Instruction instruction = 
            new ClassRefInstruction(opc_instanceof,
                                    dest.toReference().getQualifiedName());
        code.plantInstruction(instruction);
    }
}
