/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.type.*;

/**
 * JOuterLocalVariableExpression
 */
public class JOuterLocalVariableExpression
    extends JLocalVariableExpression
{
    /**
     * Class of method containing local class.
     */
    public CSourceClass outer;

    /**
     * Construct a node in the parsing tree.
     *
     * @param where
     * The line of this node in the source code.
     */
    public JOuterLocalVariableExpression(TokenReference where,
                                         JLocalVariable var,
                                         CSourceClass outer)
    {
        super(where, var);

        this.outer = outer;
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context
     * The actual context of analyse.
     * @return
     * A pure java expression including promote node.
     * @exception PositionedError
     * If the check fails.
     */
    public JExpression checkExpression(CExpressionContext context)
        throws PositionedError
    {
        /* First we have to find the right context */
        CContext body = context.getBodyContext();
        while (!body.getClassContext().getCClass().equals(outer)) {
            body = body.getClassContext().getParentContext();
        }

        CContext parent = body.getClassContext().getParentContext();
        CExpressionContext exprContext;
        if (parent instanceof CExpressionContext) {
            exprContext = (CExpressionContext)parent;
        }
        else {
            exprContext = new CExpressionContext((CBodyContext)parent);
        }

        JExpression expr = super.checkExpression(exprContext);

        if (! (expr instanceof JLiteral) ) {
            check(context,
                  expr == this && getVariable().isFinal(),
                  KjcMessages.BAD_LOCAL_NOT_FINAL,
                  getVariable().getIdent());

            CMethod method = null;
            if (context.getMethodContext() instanceof CConstructorContext) {
                CConstructorContext tmp =
                    (CConstructorContext)context.getMethodContext();
                method = tmp.getCMethod();
            }

            expr = outer.getOuterLocalAccess(getTokenReference(),
                                             getVariable(),
                                             method);
            expr.checkExpression(context);
        }

        return expr;
    }
}
