/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.type.*;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents expression within parenthese
 */
public class JParenthesedExpression
    extends JExpression
{

    /**
     * Construct a node in the parsing tree
     * This method is directly called by the parser
     * @param where  the line of this node in the source code
     * @param expr  the expression beetwen parenthese
     */
    public JParenthesedExpression(TokenReference where, JExpression expr)
    {
        super(where);
        this.expr = expr;
    }

    /**
     * Compute the type of this expression (called after parsing)
     * @return the type of this expression
     */
    public Type getType()
    {
        return expr.getType();
    }

    /**
     * Check expression and evaluate and alter context
     * @param context the actual context of analyse
     * @return  a pure java expression including promote node
     * @exception PositionedError  if the check fails
     * @exception UnpositionedError this error will be positioned soon
     */
    public JExpression checkExpression(CExpressionContext context)
        throws PositionedError
    {
        expr = expr.checkExpression(context);
        if (expr.isConstant() || expr instanceof JThisExpression) {
            return expr;
        }

        return this;
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitParenthesedExpression(this, expr);
    }

    /**
     * Generates a sequence of bytecodes
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        expr.genCode(code);
    }

    /**
     * Generates a sequence of bytecodes to branch on a label
     * This method helps to handle heavy optimizables conditions
     * @param code  the code list
     */
    public void genBranch(boolean cond, CodeSequence code, CodeLabel label)
    {
        expr.genBranch(cond, code, label);
    }

    /**
     * Generates a sequence of bytecodes to branch on a label
     * This method helps to handle heavy optimizables conditions
     * @param code  the code list
     */
    public JExpression unParenthesize()
    {
        return expr.unParenthesize();
    }

    private JExpression expr;
}
