/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.type.*;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * A System. expression (for static invocations)
 */
public class JTypeNameExpression
    extends JExpression
{

    /**
     * Resolved class type for name.
     */
    private ClassOrInterfaceType type;

    /**
     * Construct a node in the parsing tree.
     *
     * @param where the line of this node in the source code.
     */
    public JTypeNameExpression(TokenReference where, String qualifiedName)
    {
        super(where);

        type = ClassOrInterfaceType.lookup(qualifiedName);
    }

    /**
     * Construct a node in the parsing tree
     * @param where the line of this node in the source code
     */
    public JTypeNameExpression(TokenReference where, ClassOrInterfaceType type)
    {
        super(where);

        this.type = type;
    }

    /**
     * Compute the type of this expression (called after parsing)
     * @return the type of this expression
     */
    public Type getType()
    {
        return type;
    }

    /**
     * Compute the type of this expression (called after parsing)
     * @return the type of this expression
     */
    public ClassOrInterfaceType getClassType()
    {
        return type;
    }

    /**
     * Returns a qualified name for the type of this
     */
    public String getQualifiedName()
    {
        return type.getQualifiedName();
    }

    /**
     * Check expression and evaluate and alter context
     * @param context the actual context of analyse
     * @return  a pure java expression including promote node
     * @exception PositionedError  if the check fails
     */
    public JExpression checkExpression(CExpressionContext context)
        throws PositionedError
    {
        try {
            type.resolveType(context);
        }
        catch (UnpositionedError e) {
            throw e.addPosition(getTokenReference());
        }

        return this;
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitTypeNameExpression(this, type);
    }

    /**
     * Generates a sequence of bytecodes
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        code.setLineNumber(getTokenReference().getLine());

        // do nothing here
    }

}
