/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * Grammar based on:
 * Java 1.2 Recognizer
 *
 * Contributing authors:
 *              John Mitchell           johnm@non.net
 *              Terence Parr            parrt@magelang.com
 *              John Lilley             jlilley@empathy.com
 *              Scott Stanchfield       thetick@magelang.com
 *              Markus Mohnen           mohnen@informatik.rwth-aachen.de
 *              Peter Williams          pwilliams@netdynamics.com
 *
 * Version 1.00 December 9, 1997 -- initial release
 * Version 1.01 December 10, 1997
 *              fixed bug in octal def (0..7 not 0..8)
 * Version 1.10 August 1998 (parrt)
 *              added tree construction
 *              fixed definition of WS,comments for mac,pc,unix newlines
 *              added unary plus
 * Version 1.11 (Nov 20, 1998)
 *              Added "shutup" option to turn off last ambig warning.
 *              Fixed inner class def to allow named class defs as statements
 *              synchronized requires compound not simple statement
 *              add [] after builtInType DOT class in primaryExpression
 *              "const" is reserved but not valid..removed from modifiers
 * Version 1.12 (Feb 2, 1999)
 *              Changed LITERAL_xxx to xxx in tree grammar.
 *              Updated java.g to use tokens {...} now for 2.6.0 (new feature).
 *
 * Version 1.13 (Apr 23, 1999)
 *              Didn't have (stat)? for else clause in tree parser.
 *              Didn't gen ASTs for interface extends.  Updated tree parser too.
 *              Updated to 2.6.0.
 * Version 1.14 (Jun 20, 1999)
 *              Allowed final/abstract on local classes.
 *              Removed local interfaces from methods
 *              Put instanceof precedence where it belongs...in relationalExpr
 *                      It also had expr not type as arg; fixed it.
 *              Missing ! on SEMI in classBlock
 *              fixed: (expr) + "string" was parsed incorrectly (+ as unary plus).
 *              fixed: didn't like Object[].class in parser or tree parser
 * Version 1.15 (Jun 26, 1999)
 *              Screwed up rule with instanceof in it. :(  Fixed.
 *              Tree parser didn't like (expr).something; fixed.
 *              Allowed multiple inheritance in tree grammar. oops.
 * Version 1.16 (August 22, 1999)
 *              Extending an interface built a wacky tree: had extra EXTENDS.
 *              Tree grammar didn't allow multiple superinterfaces.
 *              Tree grammar didn't allow empty var initializer: {}
 *
 * This grammar is in the PUBLIC DOMAIN
 *
 * BUGS
 *
 */

header {
package at.dms.kjc;
}

{
import java.util.ArrayList;
import java.util.List;
  
import at.dms.compiler.Compiler;
import at.dms.compiler.CompilerMessages;
import at.dms.compiler.CWarning;
import at.dms.compiler.InputBuffer;
import at.dms.compiler.JavaStyleComment;
import at.dms.compiler.JavadocComment;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.type.*;
import at.dms.util.Utils;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class KjcParser extends Parser;
options {
    k = 2;                          // two token lookahead
    exportVocab = Kjc;
    codeGenMakeSwitchThreshold = 2; // Some optimizations
    codeGenBitsetTestThreshold = 3;
    defaultErrorHandler = false;    // Don't generate parser error handlers
}

{
    /**
     * Compiler instance.
     */
    private Compiler compiler;

    /**
     * The scanner used.
     */
    private KjcScanner scanner;

    public KjcParser(Compiler compiler, KjcScanner scanner)
    {
        super(scanner, 2);
        this.compiler = compiler;
        this.scanner = scanner;
    }

    /**
     * Returns the compiler driver which invoked the parser.
     */
    public Compiler getCompiler()
    {
        return compiler;
    }

    /**
     * Returns the input buffer.
     */
    public final InputBuffer getBuffer()
    {
        return scanner.getBuffer();
    }

    /**
     * Returns a reference to the current position in the source file.
     */
    protected final TokenReference buildTokenReference()
    {
        return scanner.getTokenReference();
    }

    /**
     *
     */
    protected final JavaStyleComment[] getStatementComment()
    {
        return scanner.getStatementComment();
    }

    /**
     *
     */
    protected final JavadocComment getJavadocComment()
    {
        return scanner.getJavadocComment();
    }

    /**
     *
     */
    protected List getComment()
    {
        return null; // scanner.getComment();
    }

    /**
     * Reports that an error has been detected in the lexical analyser.
     * The handling is delegated to the compiler driver.
     *
     * @param error
     * The error to report.
     */
    protected final void reportTrouble(PositionedError trouble)
    {
        compiler.reportTrouble(trouble);
    }

    /**
     * Generate an human readable error message.
     */
    public PositionedError beautifyParseError(antlr.RecognitionException e)
    {
        if (e instanceof NoViableAltException) {
            NoViableAltException nva = (NoViableAltException)e;
            Token t = nva.token;
        }

        String message = e.toString(); // can do better

        if (message == null) {
            message = "unknown";
        }
        else {
            int idx = message.indexOf(",");

            if (idx >= 0) {
                message = message.substring(idx + 1);
            }
        }

        return new PositionedError(scanner.getTokenReference(),
                                   CompilerMessages.SYNTAX_ERROR, message);
    }
}
        
// Compilation Unit: In Java, this is a single file.  This is the start
//   rule for this parser
jCompilationUnit []
    returns [JCompilationUnit self = null]
{
    TokenReference sourceRef = buildTokenReference();
    JPackageName pack = JPackageName.UNNAMED;
    CParseCompilationUnitContext context =
        CParseCompilationUnitContext.getInstance();
}
:
    // A compilation unit starts with an optional package definition
    (
        pack = jPackageDefinition[]
    |
        /* nothing */
    )
    { context.setPackage(pack); }
                
    // Next we have a series of zero or more import statements
    ( jImportDefinition[context] )*
                
    // Wrapping things up with any number of class or interface
    // definitions
    ( jTypeDefinition[context] )*
        
    EOF
    {
        self = new JCompilationUnit(sourceRef, pack,
                                    context.getPackageImports(),
                                    context.getClassImports(),
                                    context.getTypeDeclarations());
        context.release();
    }
;

// Package statement: "package" followed by an identifier.
jPackageDefinition []
    returns [JPackageName self = null]
{
    String  name = null;
}
:
    "package" name = jIdentifier[] SEMI
    {
        self = new JPackageName(buildTokenReference(),
                                name, getStatementComment());
    }
;

// Import statement: import followed by a package or class name
jImportDefinition [CParseCompilationUnitContext cunit]
{
    StringBuffer buffer = null;
    boolean      star = false;
    String       name = null;
}
:
    "import" i:IDENT  
    (
        DOT j:IDENT
        {
            ( buffer == null
              ? (buffer = new StringBuffer(i.getText()))
              : buffer
            ).append('/').append(j.getText());
        }
    )*
    {
        name = (buffer == null ? i.getText() : buffer.toString());
    }

    ( DOT STAR { star = true; } )?
    SEMI
    {
        if (star) {
            cunit.addPackageImport(new JPackageImport(buildTokenReference(),
                                                      name,
                                                      getStatementComment()));
        }
        else {
            cunit.addClassImport(new JClassImport(buildTokenReference(),
                                                  name,
                                                  getStatementComment()));
        }
    }
;

// A type definition in a file is either a class or interface definition.
jTypeDefinition [CParseCompilationUnitContext context]
{
    int mods = 0;
    JTypeDeclaration decl = null;
    TokenReference sourceRef = buildTokenReference();
}
:
    mods = jModifiers[]
    (
        decl = jClassDefinition[mods]
    |
        decl = jInterfaceDefinition[mods]
    )
    {
        context.addTypeDeclaration(getCompiler(), decl);
    }

    |

    SEMI
    {
        reportTrouble(new CWarning(sourceRef,
                                   KjcMessages.STRAY_SEMICOLON, null));
    }
;

/**
 * A declaration is the creation of a reference or primitive-type variable
 *  Create a separate Type/Var tree for each var in the var list.
 */
jDeclaration []
    returns [JVariableDeclarationStatement self = null]
{
    int mods;
    Type type;
    JVariableDefinition[] decl;
    TokenReference sourceRef = buildTokenReference();
}
:
    mods = jModifiers[]
    type = jTypeSpec[]
    decl = jVariableDefinitions[mods, type]
    { 
        self = new JVariableDeclarationStatement(sourceRef, 
                                                 decl,
                                                 getStatementComment());
    }
;

// A list of zero or more modifiers.  We could have used (modifier)* in
//   place of a call to modifiers, but I thought it was a good idea to keep
//   this rule separate so they can easily be collected in a List if
//   someone so desires
jModifiers []
    returns [int self = 0]
{
    int mod;
}
:
    ( 
        mod = jModifier[]
        {
            if ((mod & self) != 0) {
                reportTrouble(new PositionedError
                    (buildTokenReference(),
                     KjcMessages.DUPLICATE_MODIFIER,
                     JMemberDeclaration.getModifierName(mod)));
            }

            if (!JMemberDeclaration.checkOrder(self, mod)) {
                reportTrouble(new CWarning
                    (buildTokenReference(),
                     KjcMessages.MODIFIER_ORDER,
                     JMemberDeclaration.getModifierName(mod)));
            }
            self |= mod;
        }
    )*
;


// A type specification is a type name with possible brackets afterwards
// (which would make it an array type).
jTypeSpec []
    returns [Type self = null]
:
    self = jClassTypeSpec[]
    | self = jBuiltInTypeSpec[]
;

// A class type specification is a class type with possible brackets afterwards
// (which would make it an array type).
jClassTypeSpec []
    returns [ReferenceType self = null]
{
    String name = null;
    int bounds = 0;
}
:
    name = jIdentifier[]
    { self = ClassOrInterfaceType.lookup(name); }
    ( 
        LBRACK RBRACK { bounds += 1; } 
    )*
    {
        if (bounds > 0) {
            self = new ArrayType(self, bounds);
        }
    }
;

// A builtin type specification is a builtin type with possible brackets
// afterwards (which would make it an array type).
jBuiltInTypeSpec []
    returns [Type self = null]
{
    int bounds = 0;
}
:
    self = jBuiltInType[]
    ( 
        LBRACK RBRACK { bounds += 1; } 
    )*
    {
        if (bounds > 0) {
            self = new ArrayType(self, bounds);
        }
    }
;

jType []
    returns [Type type = null]
{
    String name;
}
:
    type = jBuiltInType[]
    | name = jIdentifier[] { type = ClassOrInterfaceType.lookup(name); }
;

// !!!JLS The primitive types.
jBuiltInType []
    returns [Type self = null]
:
    "boolean"  { self = StandardType.BOOLEAN; }
    | "byte"   { self = StandardType.BYTE; }
    | "char"   { self = StandardType.CHAR; }
    | "double" { self = StandardType.DOUBLE; }
    | "float"  { self = StandardType.FLOAT; }
    | "int"    { self = StandardType.INTEGER; }
    | "long"   { self = StandardType.LONG; }
    | "short"  { self = StandardType.SHORT; }
    | "void"   { self = StandardType.VOID; }
;


// A (possibly-qualified) java identifier.  We start with the first IDENT
//   and expand its name by adding dots and following IDENTS
jIdentifier []
    returns [String self = null]
{
    StringBuffer buffer = null;
}
:
    i:IDENT  
    (
        DOT j:IDENT
        {
            (buffer == null
             ? (buffer = new StringBuffer(i.getText()))
             : buffer).append('/').append(j.getText());
        }
    )*
    {
        self = (buffer == null ? i.getText() : buffer.toString());
    }
;

// modifiers for Java classes, interfaces, class/instance vars and methods
jModifier []
    returns [int self = 0]
:
    "public"         { self = at.dms.kjc.Constants.ACC_PUBLIC; }
    | "protected"    { self = at.dms.kjc.Constants.ACC_PROTECTED; }
    | "private"      { self = at.dms.kjc.Constants.ACC_PRIVATE; }
    | "static"       { self = at.dms.kjc.Constants.ACC_STATIC; }
    | "abstract"     { self = at.dms.kjc.Constants.ACC_ABSTRACT; }
    | "final"        { self = at.dms.kjc.Constants.ACC_FINAL; }
    | "native"       { self = at.dms.kjc.Constants.ACC_NATIVE; }
    | "synchronized" { self = at.dms.kjc.Constants.ACC_SYNCHRONIZED; }
    | "strictfp"     { self = at.dms.kjc.Constants.ACC_STRICT; }
    | "transient"    { self = at.dms.kjc.Constants.ACC_TRANSIENT; }
    | "volatile"     { self = at.dms.kjc.Constants.ACC_VOLATILE; }
;
        

// Definition of a Java class
jClassDefinition[int mods]
    returns [JClassDeclaration self = null]
{
    String superClass = null;
    ClassOrInterfaceType[] interfaces = ClassOrInterfaceType.EMPTY;
    CParseClassContext context = CParseClassContext.getInstance();
    TokenReference sourceRef = buildTokenReference();
    JavadocComment javadoc = getJavadocComment();
    JavaStyleComment[] comments = getStatementComment();
}
:
    "class" ident:IDENT
    // it _might_ have a superclass...
    superClass = jSuperClassClause[]
    // it might implement some interfaces...
    interfaces = jImplementsClause[]
    // now parse the body of the class
    jClassBlock[context]
    {
        self = new JClassDeclaration(sourceRef,
                                     mods,
                                     ident.getText(),
                                     superClass,
                                     interfaces,
                                     context.getFields(),
                                     context.getMethods(),
                                     context.getInnerClasses(),
                                     context.getBody(),
                                     javadoc,
                                     comments);
        context.release();
    }
;

jSuperClassClause []
    returns [String self = null]
:
    ( "extends" self = jIdentifier[] )?
;

// Definition of a Java Interface
jInterfaceDefinition [int mods]
  returns [JInterfaceDeclaration self = null]
{
    ClassOrInterfaceType[]       interfaces =  ClassOrInterfaceType.EMPTY;
    CParseClassContext context = CParseClassContext.getInstance();
    TokenReference     sourceRef = buildTokenReference();
}
:
    "interface" ident:IDENT
    // it might extend some other interfaces
    interfaces = jInterfaceExtends[]
    // now parse the body of the interface (looks like a class...)
    jClassBlock[context]
    {
        self = new JInterfaceDeclaration(sourceRef,
                                         mods, ident.getText(),
                                         interfaces,
                                         context.getFields(),
                                         context.getMethods(),
                                         context.getInnerClasses(),
                                         context.getBody(),
                                         getJavadocComment(),
                                         getStatementComment());
        context.release();
    }
;


// This is the body of a class.  You can have fields and extra semicolons,
// That's about it (until you see what a field is...)
jClassBlock [CParseClassContext context]
:
    LCURLY
    ( jMember[context] 
    | SEMI { reportTrouble(new CWarning(buildTokenReference(),
                                        KjcMessages.STRAY_SEMICOLON, null)); }
    )*
    RCURLY
;

// An interface can extend several other interfaces...
jInterfaceExtends []
    returns [ClassOrInterfaceType[] self = ClassOrInterfaceType.EMPTY]
:
    ( "extends" self = jNameList[] )?
;

// A class can implement several interfaces...
jImplementsClause[]
    returns [ClassOrInterfaceType[] self = ClassOrInterfaceType.EMPTY]
:
    ( "implements" self = jNameList[] )?
;

// Now the various things that can be defined inside a class or interface:
// methods, constructors, or variable declaration
// Note that not all of these are really valid in an interface (constructors,
// for example), and if this grammar were used for a compiler there would
// need to be some semantic checks to make sure we're doing the right thing...
jMember [CParseClassContext context]
{
    int mods = 0;
    JStatement[] body = null;
    JTypeDeclaration decl;
    Type type;
    JFormalParameter[] params;
    int bounds = 0;
    ClassOrInterfaceType[] throwsList = ClassOrInterfaceType.EMPTY;
    JVariableDefinition[] vars;
    TokenReference sourceRef = buildTokenReference();
}
:
    mods = jModifiers[]
    (
        // Ctor
        ctor:IDENT
        // parse the formal parameter declarations.
        LPAREN
        params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER]
        RPAREN

        // get the list of exceptions that this method is declared to throw
        ( throwsList = jThrowsClause[] )?
        body = jCompoundStatement[]
        {
            context.addMethodDeclaration
                (new JConstructorDeclaration(sourceRef,
                                             mods,
                                             ctor.getText(),
                                             params,
                                             throwsList,
                                             new JConstructorBlock(sourceRef,
                                                                   body),
                                             getJavadocComment(),
                                             getStatementComment()));
        }
    |
        decl = jClassDefinition[mods]              // inner class
        {
            context.addInnerDeclaration(decl);
        }
    |
        decl = jInterfaceDefinition[mods]         // inner interface
        {
            context.addInnerDeclaration(decl);
        }
    |
        // method or variable declaration(s)
        type = jTypeSpec[]
        (
            ident:IDENT
            // parse the formal parameter declarations.
            LPAREN
            params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER]
            RPAREN
            (
                LBRACK RBRACK { bounds += 1; }
            )*
            {
                if (bounds > 0) {
                    reportTrouble
                        (new CWarning(sourceRef,
                                      KjcMessages.OLD_STYLE_ARRAY_BOUNDS,
                                      null));
                    type = new ArrayType(type, bounds);
                }
            }

            (throwsList = jThrowsClause[])?
            ( body = jCompoundStatement[] | SEMI )
            {
                context.addMethodDeclaration
                    (new JMethodDeclaration
                        (sourceRef, mods, type, ident.getText(), params,
                         throwsList,
                         ( body == null ? null
                                        : new JBlock(sourceRef, body, null)),
                         getJavadocComment(),
                         getStatementComment()));
            }
        |
            vars = jVariableDefinitions[mods, type] SEMI
            {
                for (int i = 0; i < vars.length; i++) {
                    context.addFieldDeclaration
                        (new JFieldDeclaration(sourceRef,
                                               vars[i], 
                                               getJavadocComment(),
                                               getStatementComment())); 
                }
            }
        )
    )
|
    // "static { ... }" class initializer
    "static" body = jCompoundStatement[]
    {
        context.addBlockInitializer(new JClassBlock(sourceRef, true, body));
    }
|
    // "{ ... }" instance initializer
    body = jCompoundStatement[]
    {
        context.addBlockInitializer(new JClassBlock(sourceRef, false, body));
    }
;

jVariableDefinitions [int mods, Type type]
    returns [JVariableDefinition[] self = null]
{
    List vars = new ArrayList(10);
    JVariableDefinition decl;
}
:
    decl = jVariableDeclarator[mods, type]  { vars.add(decl); }
    (     
        COMMA decl = jVariableDeclarator[mods, type]  { vars.add(decl); }
    )*
    {
        self = (JVariableDefinition[])
            vars.toArray(new JVariableDefinition[vars.size()]);
    }
;

/**
 * Declaration of a variable.  This can be a class/instance variable,
 * or a local variable in a method.
 * It can also include possible initialization.
 */
jVariableDeclarator [int mods, Type type]
    returns [JVariableDefinition self = null]
{
    int            bounds = 0;
    JExpression    expr = null;
    TokenReference sourceRef = buildTokenReference();
}
:
    ident:IDENT
    ( LBRACK RBRACK { bounds += 1; } )*
    expr = jVarInitializer[]
    {
        if (bounds > 0) {
            reportTrouble(new CWarning(sourceRef,
                                       KjcMessages.OLD_STYLE_ARRAY_BOUNDS,
                                       null));
            type = new ArrayType(type, bounds);
        }
        self = new JVariableDefinition(sourceRef, mods, type, ident.getText(),
                                       expr);
    }
;


jVarInitializer []
    returns [JExpression self = null]
:
    ( ASSIGN self = jInitializer[] )?
;

// This is an initializer used to set up an array.
jArrayInitializer[]
    returns [JArrayInitializer self = null]
{
    List list = new ArrayList(10);
    JExpression    expr = null;
    TokenReference sourceRef = buildTokenReference();
}
:
    LCURLY
    (COMMA | (list = jInitializers[list])? )
    RCURLY
    {
        self =
            new JArrayInitializer(sourceRef, (JExpression[])
                                  list.toArray(new JExpression[list.size()])); 
    }
;

// See JLS 10.6, "VariableInitializers"
jInitializers[List expressions]
  returns [List newExpressions = expressions]
{
    JExpression    initializer = null;
    TokenReference sourceRef = buildTokenReference();
}
:
    initializer = jInitializer[]
    {
        newExpressions.add(initializer);
    }
    // (
        // options { warnWhenFollowAmbig = false; } :
        ( COMMA (newExpressions = jInitializers[newExpressions]) ? ) ?
    // )
;

// The two "things" that can initialize an array element are an expression
//   and another (nested) array initializer.
// JLS 8.3
jInitializer []
  returns [JExpression self = null]
        :
                self = jExpression[]
        |       
                self = jArrayInitializer[]
        ;

// This is a list of exception classes that the method is declared to throw
jThrowsClause []
  returns [ClassOrInterfaceType[] self]
        :       
                "throws" self = jNameList[]
        ;


// A list of formal parameters
jParameterDeclarationList [int desc]
 returns [JFormalParameter[] self = JFormalParameter.EMPTY]
{
  JFormalParameter              elem = null;
  List                        list = new ArrayList(10);
}
:

                ( 
                        elem = jParameterDeclaration[desc] 
                          { list.add(elem); }
                        (
                                 COMMA elem = jParameterDeclaration[desc] 
                                   { list.add(elem); }
                        )*
                  {
                    self = (JFormalParameter[])list.toArray(new JFormalParameter[list.size()]); 
                  }
                )?
        ;

// A formal parameter.
jParameterDeclaration [int desc]
  returns [JFormalParameter self = null]
{
  boolean       isFinal = false;
  int           bounds = 0;
  Type         type;
  TokenReference        sourceRef = buildTokenReference();
}
:
  ( "final" { isFinal = true; } )? 
  type = jTypeSpec[] 
  ident:IDENT
  ( LBRACK RBRACK { bounds += 1; } )*
    {
      if (bounds > 0) {
        reportTrouble(new CWarning(sourceRef, KjcMessages.OLD_STYLE_ARRAY_BOUNDS, null));
        type = new ArrayType(type, bounds);
      }
      self = new JFormalParameter(sourceRef, desc, type, ident.getText(), isFinal);
    }
;

// Compound statement.  This is used in many contexts:
//   Inside a class definition prefixed with "static":
//      it is a class initializer
//   Inside a class definition without "static":
//      it is an instance initializer
//   As the body of a method
//   As a completely indepdent braced block of code inside a method
//      it starts a new scope for variable definitions

jCompoundStatement[]
  returns [JStatement[] self = null]
{
  List                body = new ArrayList(100);
  JStatement            stmt;
}
:
  LCURLY
  // include the (possibly-empty) list of statements
  (
   stmt = jStatement[]
     {
       if (stmt instanceof JEmptyStatement) {
         reportTrouble(new CWarning(stmt.getTokenReference(),
                                    KjcMessages.STRAY_SEMICOLON, null));
       }
       body.add(stmt); }
  )*
  RCURLY
    {
      self = (JStatement[])body.toArray(new JStatement[body.size()]);
    }
;

jStatement[]
   returns [JStatement self = null]
{
  JExpression           expr;
  JStatement[]          stmts;
  JTypeDeclaration      type;
  JavaStyleComment[]    comments = getStatementComment();
  TokenReference        sourceRef = buildTokenReference();
}
:
  // A list of statements in curly braces -- start a new scope!
  stmts = jCompoundStatement[]
    { self = new JBlock(sourceRef, stmts, comments); }
|
  // class definition
  type = jClassDefinition[0]
    { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
|
  // final class definition
  "final" type = jClassDefinition[at.dms.kjc.Constants.ACC_FINAL]
    { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
|
  // abstract class definition
  "abstract" type = jClassDefinition[at.dms.kjc.Constants.ACC_ABSTRACT]
    { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
|
  // declarations are ambiguous with "ID DOT" relative to expression
  // statements.  Must backtrack to be sure.  Could use a semantic
  // predicate to test symbol table to see what the type was coming
  // up, but that's pretty hard without a symbol table ;)
  (jModifiers[] jTypeSpec[] IDENT) =>
  self = jDeclaration[] SEMI
|
  // An expression statement.  This could be a method call,
  // assignment statement, or any other expression evaluated for
  // side-effects.
  expr = jExpression[] SEMI
    { self = new JExpressionStatement(sourceRef, expr, comments); }
|
  self = jLabeledStatement[]
|
  self = jIfStatement[]
|
  self = jForStatement[]
|
  self = jWhileStatement[]
|
  self = jDoStatement[]
|
  self = jBreakStatement[]
|
  self = jContinueStatement[]
|
  self = jReturnStatement[]
|
  self = jSwitchStatement[]
|
  // exception try-catch block
  self = jTryBlock[]
|
  self = jThrowStatement[]
|
  self = jSynchronizedStatement[]
|
  // empty statement
  SEMI
    { self = new JEmptyStatement(sourceRef, comments); }
;

jLabeledStatement []
  returns [JLabeledStatement self = null]
{
  JStatement            stmt;
  TokenReference        sourceRef = buildTokenReference();
}
:
  // Attach a label to the front of a statement
  label:IDENT COLON stmt = jStatement[]
    {
      if (stmt instanceof JEmptyStatement) {
        reportTrouble(new CWarning(sourceRef,
                                   KjcMessages.STRAY_SEMICOLON, null));
      }
      if (stmt instanceof JVariableDeclarationStatement) {
        reportTrouble(new PositionedError(sourceRef,
                                          KjcMessages.LABEL_NOT_STATEMENT,
                                          null));
      }
      self = new JLabeledStatement(sourceRef, label.getText(),
                                   stmt, getStatementComment());
    }
;

jIfStatement []
  returns [JIfStatement self = null]
{
  JExpression           cond;
  JStatement            thenClause;
  JStatement            elseClause = null;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "if" LPAREN cond = jExpression[] RPAREN
  thenClause = jStatement[]
/*
  {
    // LOOKAHEAD OF 2 FORBIDDEN
    if (LA(1)==LITERAL_else) {
      match(LITERAL_else);
      elseClause = jStatement();
    }
  }
*/
  (
    // CONFLICT: the old "dangling-else" problem...
    // ANTLR generates proper code matching as soon as possible,
    // binding dangling else's to the innermost if statement
    // Hush warning.
    options { warnWhenFollowAmbig = false; } :
      "else" 
        elseClause = jStatement[]
  )?
    {
      if (! (thenClause instanceof JBlock)) {
        reportTrouble(new CWarning(sourceRef, KjcMessages.ENCLOSE_IF_THEN_IN_BLOCK, null));
      }
      if (elseClause != null && !(elseClause instanceof JBlock || elseClause instanceof JIfStatement)) {
        reportTrouble(new CWarning(sourceRef, KjcMessages.ENCLOSE_IF_ELSE_IN_BLOCK, null));
      }
      self = new JIfStatement(sourceRef, cond, thenClause, elseClause, getStatementComment());
    }
;

jWhileStatement []
  returns [JWhileStatement self = null]
{
  JExpression           cond;
  JStatement            body;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "while" LPAREN cond = jExpression[] RPAREN body = jStatement[]
    {
      if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
        reportTrouble(new CWarning(sourceRef, KjcMessages.ENCLOSE_LOOP_BODY_IN_BLOCK, null));
      }
      self = new JWhileStatement(sourceRef, cond, body, getStatementComment());
    }
;

jDoStatement []
  returns [JDoStatement self = null]
{
  JExpression           cond;
  JStatement            body;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "do" body = jStatement[] "while" LPAREN cond = jExpression[] RPAREN SEMI
    {
      if (! (body instanceof JBlock)) {
        reportTrouble(new CWarning(sourceRef, KjcMessages.ENCLOSE_LOOP_BODY_IN_BLOCK, null));
      }
      self = new JDoStatement(sourceRef, cond, body, getStatementComment());
    }
;

jBreakStatement []
  returns [JBreakStatement self = null]
{
  TokenReference        sourceRef = buildTokenReference();
}
:
  "break" ( label:IDENT )? SEMI
    { self = new JBreakStatement(sourceRef, label == null ? null : label.getText(), getStatementComment()); }
;

jContinueStatement []
  returns [JContinueStatement self = null]
{
  TokenReference        sourceRef = buildTokenReference();
}
:
  "continue" ( label:IDENT )? SEMI
    { self = new JContinueStatement(sourceRef, label == null ? null : label.getText(), getStatementComment()); }
;

jReturnStatement []
  returns [JReturnStatement self = null]
{
  JExpression           expr = null;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "return" ( expr = jExpression[] )? SEMI
    { self = new JReturnStatement(sourceRef, expr, getStatementComment()); }
;

jThrowStatement []
  returns [JThrowStatement self = null]
{
  JExpression           expr;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "throw" expr = jExpression[] SEMI
    { self = new JThrowStatement(sourceRef, expr, getStatementComment()); }
;

jSynchronizedStatement []
  returns [JSynchronizedStatement self = null]
{
  JExpression           expr;
  JStatement[]          body;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "synchronized" LPAREN expr = jExpression[] RPAREN body = jCompoundStatement[]
    { 
      self = new JSynchronizedStatement(sourceRef,
                                        expr, 
                                        new JBlock(sourceRef, body, null),
                                        getStatementComment());
    }
;

jSwitchStatement []
  returns [JSwitchStatement self = null]
{
    JExpression           expr = null;
    List                body = null;
    JSwitchGroup          group;
    TokenReference        sourceRef = buildTokenReference();
}
:
    "switch"
    LPAREN expr = jExpression[] RPAREN
    LCURLY
    { body = new ArrayList(10); }
    ( group = jCasesGroup[]  { body.add(group); } )*
    RCURLY
    { 
        self =
            new JSwitchStatement(sourceRef,
                                 expr, 
                                 (JSwitchGroup[])
                                 body.toArray(new JSwitchGroup[body.size()]),
                                 getStatementComment());
    }
;

jCasesGroup []
    returns [JSwitchGroup self = null]
{
    List labels = new ArrayList(10);
    List stmts = new ArrayList(10);

    JSwitchLabel          label;
    JStatement            stmt;
    TokenReference        sourceRef = buildTokenReference();
}
:
    (
        // CONFLICT: to which case group do the statements bind?
        //           ANTLR generates proper code: it groups the
        //           many "case"/"default" labels together then
        //           follows them with the statements
        options {
            warnWhenFollowAmbig = false;
        } :
        label = jACase[] { labels.add(label); }
    )+
    (
        stmt = jStatement[]
        {
            if (stmt instanceof JEmptyStatement) {
                reportTrouble(new CWarning(stmt.getTokenReference(),
                                           KjcMessages.STRAY_SEMICOLON, null));
            }
            stmts.add(stmt);
        }
    )*
    {
        self =
            new JSwitchGroup(sourceRef,
                             (JSwitchLabel[])
                             labels.toArray(new JSwitchLabel[labels.size()]), 
                             (JStatement[])
                             stmts.toArray(new JStatement[stmts.size()]));
    }
;

jACase []
  returns [JSwitchLabel self = null]
{
  JExpression           expr = null;
  TokenReference        sourceRef = buildTokenReference();
}
:
  (
    "case" expr = jExpression[] 
      { self = new JSwitchLabel(sourceRef, expr); }
  |
    "default"
      { self = new JSwitchLabel(sourceRef, null); }
  ) COLON
;


jForStatement []
  returns [JForStatement self = null]
{
  JStatement            init;
  JExpression           cond;
  JStatement            iter;
  JStatement            body;
  TokenReference        sourceRef = buildTokenReference();
}
:
  "for"
  LPAREN
  init = jForInit[] SEMI
  cond = jForCond[] SEMI
  iter = jForIter[]
  RPAREN
  body = jStatement[]
    {
      if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
        reportTrouble(new CWarning(sourceRef, KjcMessages.ENCLOSE_LOOP_BODY_IN_BLOCK, null));
      }
      self = new JForStatement(sourceRef, init, cond, iter, body, getStatementComment());
    }
;

// The initializer for a for loop
jForInit []
  returns [JStatement self = null]
{
  JExpression[] list;
}
:
(
  // if it looks like a declaration, it is
  (jDeclaration[])=> self = jDeclaration[]
|
  // otherwise it could be an expression list...
  list = jExpressionList[]
    { self = new JExpressionListStatement(buildTokenReference(), list, getStatementComment()); }
)?
;

jForCond []
  returns [JExpression expr = null]
:
  ( expr = jExpression[] )?
;

jForIter []
  returns [JExpressionListStatement self = null]
{
  JExpression[] list;
}
:
  (
   list = jExpressionList[]
     { self = new JExpressionListStatement(buildTokenReference(), list, null); }
  )?
;

// an exception handler try/catch block
jTryBlock []
  returns [JStatement self = null]
{
  JBlock                tryClause = null;
  JStatement[]          compound;
  List                catchClauses = new ArrayList(10);
  JBlock                finallyClause = null;
  JCatchClause          catcher = null;
  TokenReference        sourceRef = buildTokenReference();
}
:
        "try" compound = jCompoundStatement[]
                  { tryClause = new JBlock(sourceRef, compound, null); }
                (
                        catcher = jCatchHandler[]
                          { catchClauses.add(catcher); }
                )*
                ( 
                        "finally" compound = jCompoundStatement[] 
                          { finallyClause = new JBlock(sourceRef, compound, null); }
                )?
                  {
                    if (catchClauses.size() > 0) {
                      self = new JTryCatchStatement(sourceRef,
                                                    tryClause,
                                                    (JCatchClause[])catchClauses.toArray(new JCatchClause[catchClauses.size()]),
                                                    finallyClause == null ? getStatementComment() : null);
                    }
                    if (finallyClause != null) {
                      // If both catch and finally clauses are present,
                      // the try-catch is embedded as try clause into a
                      // try-finally statement.
                      if (self != null) {
                        tryClause = new JBlock(sourceRef, new JStatement[] {self}, null);
                      }
                      self = new JTryFinallyStatement(sourceRef, tryClause, finallyClause, getStatementComment());
                    }
                    
                    if (self == null) {
                      // try without catch or finally: error
                      reportTrouble(new PositionedError(sourceRef, KjcMessages.TRY_NOCATCH, null));
                      self = tryClause;
                    }
                  }
        ;


// an exception handler
jCatchHandler []
  returns [JCatchClause self = null]
{
    JFormalParameter      param;
    JStatement[]          body;
    TokenReference        sourceRef = buildTokenReference();
}
:
    "catch" LPAREN
    param = jParameterDeclaration[JLocalVariable.DES_CATCH_PARAMETER]
    RPAREN body = jCompoundStatement[]
    { 
        self = new JCatchClause(sourceRef, param,
                                new JBlock(sourceRef, body, null)); 
    }
;


// expressions
// Note that most of these expressions follow the pattern
//   thisLevelExpression :
//       nextHigherPrecedenceExpression
//           (OPERATOR nextHigherPrecedenceExpression)*
// which is a standard recursive definition for a parsing an expression.
// The operators in java have the following precedences:
//    lowest  (13)  = *= /= %= += -= <<= >>= >>>= &= ^= |=
//            (12)  ?:
//            (11)  ||
//            (10)  &&
//            ( 9)  |
//            ( 8)  ^
//            ( 7)  &
//            ( 6)  == !=
//            ( 5)  < <= > >=
//            ( 4)  << >>
//            ( 3)  +(binary) -(binary)
//            ( 2)  * / %
//            ( 1)  ++ -- +(unary) -(unary)  ~  !  (type)
//                  []   () (method call)  . (dot -- identifier qualification)
//                  new   ()  (explicit parenthesis)
//
// the last two are not usually on a precedence chart; I put them in
// to point out that new has a higher precedence than '.', so you
// can validy use
//     new Frame().show()
// 
// Note that the above precedence levels map to the rules below...
// Once you have a precedence chart, writing the appropriate rules as below
//   is usually very straightfoward



// the mother of all expressions
jExpression []
    returns [JExpression self = null]
:
    self = jAssignmentExpression[]
;


// This is a list of expressions.
jExpressionList []
    returns [JExpression[] self = null]
{
    JExpression expr;
    List list = new ArrayList(10);
}
:
    expr = jExpression[] { list.add(expr); }
    ( COMMA expr = jExpression[] { list.add(expr); } )*
    {
        self = (JExpression[])list.toArray(new JExpression[list.size()]);
    }
;


// assignment expression (level 13)
jAssignmentExpression []
    returns [JExpression self = null]
{
    int oper = -1;
    JExpression right;
}
:
    self = jConditionalExpression[]
    (       
        (   ASSIGN { oper = Constants.OPE_SIMPLE; }
        |   PLUS_ASSIGN { oper = Constants.OPE_PLUS; }
        |   MINUS_ASSIGN { oper = Constants.OPE_MINUS; }
        |   STAR_ASSIGN { oper = Constants.OPE_STAR; }
        |   SLASH_ASSIGN { oper = Constants.OPE_SLASH; }
        |   PERCENT_ASSIGN { oper = Constants.OPE_PERCENT; }
        |   SR_ASSIGN { oper = Constants.OPE_SR; }
        |   BSR_ASSIGN { oper = Constants.OPE_BSR; }
        |   SL_ASSIGN { oper = Constants.OPE_SL; }
        |   BAND_ASSIGN { oper = Constants.OPE_BAND; }
        |   BXOR_ASSIGN { oper = Constants.OPE_BXOR; }
        |   BOR_ASSIGN { oper = Constants.OPE_BOR; }
        )
    right = jAssignmentExpression[]
    {
        self = JAssignmentExpression.create(self.getTokenReference(),
                                            oper, self, right);
    }
    )?
;


// conditional test (level 12)
jConditionalExpression []
  returns [JExpression self = null]
{
  JExpression           middle;
  JExpression           right;
}
:
        self = jLogicalOrExpression[]
                ( 
                        QUESTION middle = jConditionalExpression[] COLON right = jConditionalExpression[] 
                          { self = new JConditionalExpression(self.getTokenReference(), self, middle, right); }
                )?
        ;


// logical or (||)  (level 11)
jLogicalOrExpression []
  returns [JExpression self = null]
{
  JExpression           right;
}
:
        self = jLogicalAndExpression[] 
                (
                        LOR right = jLogicalAndExpression[]
                          { self = new JConditionalOrExpression(self.getTokenReference(), self, right); }
                )*
        ;


// logical and (&&)  (level 10)
jLogicalAndExpression []
  returns [JExpression self = null]
{
  JExpression           right;
}
:
        self = jInclusiveOrExpression[] 
                (
                        LAND right = jInclusiveOrExpression[]
                          { self = new JConditionalAndExpression(self.getTokenReference(), self, right); }
                )*
        ;


// bitwise or non-short-circuiting or (|)  (level 9)
jInclusiveOrExpression []
  returns [JExpression self = null]
{
  JExpression           right;
}
:
        self = jExclusiveOrExpression[] 
                (
                        BOR right = jExclusiveOrExpression[]
                          { self = new JBitwiseExpression(self.getTokenReference(), Constants.OPE_BOR, self, right); }
                )*
        ;


// exclusive or (^)  (level 8)
jExclusiveOrExpression []
  returns [JExpression self = null]
{
  JExpression           right;
}
:
        self = jAndExpression[] 
                (
                        BXOR right = jAndExpression[]
                          { self = new JBitwiseExpression(self.getTokenReference(), Constants.OPE_BXOR, self, right); }
                )*
        ;


// bitwise or non-short-circuiting and (&)  (level 7)
jAndExpression []
  returns [JExpression self = null]
{
  JExpression           right;
}
:
        self = jEqualityExpression[] 
                (
                        BAND right = jEqualityExpression[]
                          { self = new JBitwiseExpression(self.getTokenReference(), Constants.OPE_BAND, self, right); }
                )*
        ;


// equality/inequality (==/!=) (level 6)
jEqualityExpression []
  returns [JExpression self = null]
{
  int                   operator = -1;
  JExpression           right;
}
:
        self = jRelationalExpression[] 
                (
                        (
                                NOT_EQUAL { operator = Constants.OPE_NE; }
                        |
                                EQUAL { operator = Constants.OPE_EQ; }
                        ) 
                        right = jRelationalExpression[]
                          { self = new JEqualityExpression(self.getTokenReference(), operator, self, right); }
                )*
        ;


// boolean relational expressions (level 5)
jRelationalExpression []
  returns [JExpression self = null]
{
  int                   operator = -1;
  JExpression           right;
  Type                 type;
}
:
        self = jShiftExpression[]
                (       (       (       LT { operator = Constants.OPE_LT; }
                                |       GT { operator = Constants.OPE_GT; }
                                |       LE { operator = Constants.OPE_LE; }
                                |       GE { operator = Constants.OPE_GE; }
                                )
                                right = jShiftExpression[]
                                  { self = new JRelationalExpression(self.getTokenReference(), operator, self, right); }
                        )*
                |       "instanceof" type = jTypeSpec[]
                          { self = new JInstanceofExpression(self.getTokenReference(), self, type); }
                )
        ;


// bit shift expressions (level 4)
jShiftExpression []
  returns [JExpression self = null]
{
  int                   operator = -1;
  JExpression           right;
}
:
        self = jAdditiveExpression[] 
                (
                        (
                                SL { operator = Constants.OPE_SL; } 
                        | 
                                SR { operator = Constants.OPE_SR; }
                        | 
                                BSR { operator = Constants.OPE_BSR; }
                        )
                        right = jAdditiveExpression[]
                          { self = new JShiftExpression(self.getTokenReference(), operator, self, right); }
                )*
        ;


// binary addition/subtraction (level 3)
jAdditiveExpression []
  returns [JExpression self = null]
{
  int                   oper = Constants.OPE_PLUS;
  JExpression           right;
}
:
        self = jMultiplicativeExpression[] 
                (
                        (
                                PLUS { oper = Constants.OPE_PLUS; }
                        | 
                                MINUS { oper = Constants.OPE_MINUS; }
                        )
                        right = jMultiplicativeExpression[]
                          {
                            if (oper == Constants.OPE_PLUS) {
                              self = new JAddExpression(self.getTokenReference(), self, right);
                            } else {
                              //if (oper == Constants.OPE_MINUS)
                              self = new JMinusExpression(self.getTokenReference(), self, right);
                            }
                          }
                )*
        ;


// multiplication/division/modulo (level 2)
jMultiplicativeExpression []
  returns [JExpression self = null]
{
  int                   type = 0;
  JExpression           right;
}
:
        self = jUnaryExpression[]
                (
                        (
                                STAR { type = 1; }
                        |       
                                SLASH { type = 2; }
                        | 
                                PERCENT { type = 3; }
                        ) 
                        right = jUnaryExpression[]
                          {
                            if (type == 1) {
                              self = new JMultExpression(self.getTokenReference(), self, right);
                            } else if (type == 2) {
                              self = new JDivideExpression(self.getTokenReference(), self, right);
                            } else {
                              self = new JModuloExpression(self.getTokenReference(), self, right);
                            }
                          }
                )*
        ;

jUnaryExpression []
  returns [JExpression self = null]
{
  TokenReference        sourceRef = buildTokenReference();
}
:
  INC self = jUnaryExpression[]
    { self = new JPrefixExpression(sourceRef, Constants.OPE_PREINC, self); }
|
  DEC self = jUnaryExpression[]
    { self = new JPrefixExpression(sourceRef, Constants.OPE_PREDEC, self); }
|
  MINUS self = jUnaryExpression[]
    { self = new JUnaryExpression(self.getTokenReference(), Constants.OPE_MINUS, self); }
|
  PLUS self = jUnaryExpression[]
    { self = new JUnaryExpression(self.getTokenReference(), Constants.OPE_PLUS, self); }
|
  self = jUnaryExpressionNotPlusMinus[]
;

jUnaryExpressionNotPlusMinus []
  returns [JExpression self = null]
{
    JExpression           expr;
    Type                 dest;
    TokenReference        sourceRef = buildTokenReference();
}
:
    BNOT self = jUnaryExpression[]
    {
        self = new JUnaryExpression(self.getTokenReference(),
                                    Constants.OPE_BNOT, self);
    }
|
    LNOT self = jUnaryExpression[]
    {
        self = new JUnaryExpression(self.getTokenReference(),
                                    Constants.OPE_LNOT, self);
    }
|
    (   // subrule allows option to shut off warnings
        options {
            // "(int" ambig with postfixExpr due to lack of sequence
            // info in linear approximate LL(k).  It's ok.  Shut up.
            generateAmbigWarnings=false;
        } 
        :
        // If typecast is built in type, must be numeric operand
        // Also, no reason to backtrack if type keyword like int, float...
        LPAREN dest = jBuiltInTypeSpec[] RPAREN expr = jUnaryExpression[]
        { self = new JCastExpression(sourceRef, expr, dest); }
        // Have to backtrack to see if operator follows.  If no operator
        // follows, it's a typecast.  No semantic checking needed to parse.
        // if it _looks_ like a cast, it _is_ a cast; else it's a "(expr)"
    |
        (LPAREN jClassTypeSpec[] RPAREN jUnaryExpressionNotPlusMinus[])=>
        LPAREN dest = jClassTypeSpec[] RPAREN
        expr = jUnaryExpressionNotPlusMinus[]
        { self = new JCastExpression(sourceRef, expr, dest); }
    |
        self = jPostfixExpression[]
    )
;

// qualified names, array expressions, method invocation, post inc/dec
jPostfixExpression[]
    returns [JExpression self = null]
{
    int           bounds = 0;
    JExpression   expr;
    JExpression[] args = null;
    Type         type;
}
:
    self = jPrimaryExpression[] // start with a primary
    (
        // qualified id (id.id.id.id...) -- build the name
        DOT
        (
            ident:IDENT
            {
                self = new JNameExpression(buildTokenReference(),
                                           self, ident.getText());
            }
        |
            "this"
            {
                self = new JThisExpression(buildTokenReference(), self);
            }
        |
            "class"
            {
                self = new JClassExpression(buildTokenReference(), self, 0);
            }
        |
            self = jQualifiedNewExpression[self]
        |
            "super"
            {
                self = new JSuperExpression(buildTokenReference(), self);
            }
        )
        // the above line needs a semantic check to make sure "class"
        // is the _last_ qualifier. !!!

        // allow ClassName[].class
    |
        ( LBRACK RBRACK { bounds++; } )+ DOT "class"
        {
            self = new JClassExpression(buildTokenReference(), self, bounds);
        }
    // an array indexing operation
    |
        LBRACK expr = jExpression[] RBRACK
        {
            self = new JArrayAccessExpression(buildTokenReference(),
                                              self, expr);
        }
    // method invocation
    // The next line is not strictly proper; it allows x(3)(4) or
    // x[2](4) which are not valid in Java.  If this grammar were used
    // to validate a Java program a semantic check would be needed, or
    // this rule would get really ugly...
    |
        LPAREN args = jArgList[] RPAREN
        {
            if (self instanceof JSuperExpression) {
                JExpression prefix = ((JThisExpression)self).getPrefix();
                self =
                    new JExplicitConstructorInvocation(buildTokenReference(),
                                                       prefix, "super", args); 
            }
            else if (self instanceof JThisExpression) {
                JExpression prefix = ((JThisExpression)self).getPrefix();
                self =
                    new JExplicitConstructorInvocation(buildTokenReference(),
                                                       prefix, "this", args);
            }
            else {
                self = new JMethodCallExpression(buildTokenReference(),
                                                 self, args); 
            }
        }
    )*

    // possibly add on a post-increment or post-decrement.
    // allows INC/DEC on too much, but semantics can check
    (
        INC
        {
            self = new JPostfixExpression(buildTokenReference(),
                                          Constants.OPE_POSTINC, self);
        }
    |
        DEC
        {
            self = new JPostfixExpression(buildTokenReference(),
                                          Constants.OPE_POSTDEC, self);
        }
    |
        // nothing
    )

|
    // look for int.class and int[].class
    type = jBuiltInType[] ( LBRACK RBRACK { bounds++; } )* DOT "class"
    {
        type = ( bounds > 0 ? new ArrayType(type, bounds) : type );
        self = new JClassExpression(buildTokenReference(), type);
    }
;

// the basic element of an expression
jPrimaryExpression[]
    returns [JExpression self = null]
{
    JExpression qualifier;
}
:
    (
        ident:IDENT
        {
            self = new JNameExpression(buildTokenReference(), ident.getText());
        }
    |
        self = jNewExpression[]
    |
        self = jConstant[]
    |
        "super"
        {
            self = new JSuperExpression(buildTokenReference());
        }
    |
        "true"
        {
            self = new JBooleanLiteral(buildTokenReference(), true);
        }
    |
        "false"
        {
            self = new JBooleanLiteral(buildTokenReference(), false);
        }
    |
        "this"
        {
            self = new JThisExpression(buildTokenReference() );
        }
    |
        "null"
        {
            self = new JNullLiteral(buildTokenReference());
        }
    |
        LPAREN self = jAssignmentExpression[] RPAREN
        {
            self = new JParenthesedExpression(buildTokenReference(), self);
        }
    )
;

/** object instantiation.
 *  Trees are built as illustrated by the following input/tree pairs:
 *  
 *  new T()
 *  
 *  new
 *   |
 *   T --  ELIST
 *           |
 *          arg1 -- arg2 -- .. -- argn
 *  
 *  new int[]
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *  
 *  new int[] {1,2}
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR -- ARRAY_INIT
 *                                  |
 *                                EXPR -- EXPR
 *                                  |      |
 *                                  1      2
 *  
 *  new int[3]
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *                |
 *              EXPR
 *                |
 *                3
 *  
 *  new int[1][2]
 *  
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *               |
 *         ARRAY_DECLARATOR -- EXPR
 *               |              |
 *             EXPR             1
 *               |
 *               2
 *  
 */
jNewExpression []
    returns [JExpression self = null]
{
    Type              type;
    JExpression[]      args;
    JArrayInitializer  init = null;
    JClassDeclaration  decl = null;
    CParseClassContext context = null;
}
:
    "new" type = jType[]
    (
        LPAREN args = jArgList[] RPAREN 
        ( 
            { context = CParseClassContext.getInstance(); }
            jClassBlock[context]
            {
                decl = new JClassDeclaration(buildTokenReference(),
                                             0,
                                             ((ClassOrInterfaceType)type)
                                             .getQualifiedName(),
                                             null,
                                             ClassOrInterfaceType.EMPTY,
                                             context.getFields(),
                                             context.getMethods(),
                                             context.getInnerClasses(),
                                             context.getBody(),
                                             getJavadocComment(),
                                             getStatementComment());
                context.release();
            }
            {
                self = new JNewAnonymousClassExpression(buildTokenReference(),
                                                        (ClassOrInterfaceType)type,
                                                        null, null,
                                                        args, decl);
            }
        |
            {
                self = new JNewObjectExpression(buildTokenReference(),
                                                (ClassOrInterfaceType)type,
                                                null, null,
                                                args);
            }
        )
    |
        // java 1.1
        // Note: This will allow bad constructs like
        //    new int[4][][3] {exp,exp}.
        //    There needs to be a semantic check here...
        // to make sure:
        //   a) [ expr ] and [ ] are not mixed
        //   b) [ expr ] and an init are not used together
        args = jNewArrayDeclarator[] ( init = jArrayInitializer[])?
        {
            self =
                new JNewArrayExpression(buildTokenReference(),
                                        type,
                                        new JArrayDimsAndInits
                                        (buildTokenReference(),  args, init));
        }
    )
;

jQualifiedNewExpression [JExpression expr]
    returns [JExpression self = null]
{
    String             identifier;
    JExpression[]      args;
    JArrayInitializer  init = null;
    JClassDeclaration  decl = null;
    CParseClassContext context = null;
}
:
    "new" identifier = jIdentifier[]
    (
        LPAREN args = jArgList[] RPAREN 
        ( 
            { context = CParseClassContext.getInstance(); }
            jClassBlock[context]
            {
                decl = new JClassDeclaration(buildTokenReference(),
                                             0,
                                             identifier,
                                             null,
                                             ClassOrInterfaceType.EMPTY,
                                             context.getFields(),
                                             context.getMethods(),
                                             context.getInnerClasses(),
                                             context.getBody(),
                                             getJavadocComment(),
                                             getStatementComment());
                context.release();
            }
            {
                self = new JNewAnonymousClassExpression(buildTokenReference(),
                                                        null,
                                                        expr, identifier,
                                                        args, decl);
            }
        |
            {
                self = new JNewObjectExpression(buildTokenReference(),
                                                null,
                                                expr, identifier,
                                                args);
            }
        )
    )
;

jArgList[]
    returns [JExpression[] self = JExpression.EMPTY]
:
    ( self = jExpressionList[] )?
;

jNewArrayDeclarator[]
    returns [JExpression[] self = null]
{
    boolean     unspec = false;
    List        list = new ArrayList(10);
    JExpression exp  = null;
}
:
  (
    // CONFLICT:
    // newExpression is a primaryExpression which can be
    // followed by an array index reference.  This is ok,
    // as the generated code will stay in this loop as
    // long as it sees an LBRACK (proper behavior)
    options { warnWhenFollowAmbig = false; } :
    LBRACK
    (
      { !unspec }?
      exp = jExpression[] 
      | 
      { unspec = true; }
    )
    RBRACK
      { list.add(exp); exp = null; }
  )+
    {
      self = (JExpression[])list.toArray(new JExpression[list.size()]);
    }
;

jConstant []
  returns [JExpression self = null]
:
  i:INTEGER_LITERAL
    { self = new JOrdinalLiteral(buildTokenReference(), i.getText()); }
|
  c:CHARACTER_LITERAL
    { self = new JCharLiteral(buildTokenReference(), c.getText()); }
|
  s:STRING_LITERAL
    { self = new JStringLiteral(buildTokenReference(), s.getText()); }
|
  r:REAL_LITERAL
    { self = new JRealLiteral(buildTokenReference(), r.getText()); }
;


jNameList []
  returns [ClassOrInterfaceType[] self = null]
{
  String        name;
  List        list = new ArrayList(10);
}
:
  name = jIdentifier[] { list.add(ClassOrInterfaceType.lookup(name)); }
  (
    COMMA 
    name = jIdentifier[] { list.add(ClassOrInterfaceType.lookup(name)); } 
  )*
    {
      self = (ClassOrInterfaceType[])list.toArray(new ClassOrInterfaceType[list.size()]);
    }
;

jReservedKeyword []
  returns [int self = 0]
{
}
:
    "const" 
    | "goto"
;

