/*
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.type;

import at.dms.kjc.CClass;

/**
 * Abstraction of reference types.
 * See JLS2 4.3 for a description of reference types.
 */
public abstract class ReferenceType
    extends Type
{
    /**
     * Construct a reference type.
     */
    ReferenceType(int typeID)
    {
        super(typeID);
    }

    /**
     * Returns type equality.
     */
    public boolean equalsType(Type other)
    {
        if ( other.isReference() ) {
            return equalsReferenceType((ReferenceType)other);
        }
        else {
            return false;
        }
    }

    /**
     * Returns reference type equality.
     */
    public abstract boolean equalsReferenceType(ReferenceType other);

    /**
     * Returns qualified name.
     */
    public abstract String getQualifiedName();

    /**
     * Returns the CClass object associated with this type.
     * This method should not be called before the type has been resolved.
     *
     * @return
     * The CClass object associated with this type.
     */
    public abstract CClass getCClass();

    /**
     * Returns whether this type is a reference type.
     */
    public boolean isReference()
    {
        return true;
    }
}
