/*
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.type;

import at.dms.kjc.Constants;

/**
 * Standard type declarations.
 */
public class StandardType
{
    /**
     * Primitive type 'void'.
     */
    public static final VoidType VOID;

    /**
     * Internal reference type for 'null'.
     */
    public static final NullType NULL;

    /**
     * Primitive type 'byte'.
     */
    public static final ByteType BYTE;

    /**
     * Primitive type 'int'.
     */
    public static final IntType INTEGER;

    /**
     * Primitive type 'long'.
     */
    public static final LongType LONG;

    /**
     * Primitive type 'double'.
     */
    public static final DoubleType DOUBLE;

    /**
     * Primitive type 'float'.
     */
    public static final FloatType FLOAT;

    /**
     * Primitive type 'short'.
     */
    public static final ShortType SHORT;

    /**
     * Primitive type 'char'.
     */
    public static final CharType CHAR;

    /**
     * Primitive type 'boolean'.
     */
    public static final BooleanType BOOLEAN;

    /**
     * Class type 'java.lang.Object'.
     */
    public static final ClassOrInterfaceType OBJECT;

    /**
     * Interface type 'java.lang.Cloneable'.
     */
    public static final ClassOrInterfaceType CLONEABLE;

    /**
     * Interface type 'java.io.Serializable'
     */
    public static final ClassOrInterfaceType SERIALIZABLE;

    /**
     * Class type 'java.lang.Class'.
     */
    public static final ClassOrInterfaceType CLASS;

    /**
     * Class type 'java.lang.String'.
     */
    public static final ClassOrInterfaceType STRING;

    /**
     * Class type 'java.lang.Throwable'.
     */
    public static final ClassOrInterfaceType THROWABLE;

    /**
     * Class type 'java.lang.Exception'.
     */
    public static final ClassOrInterfaceType EXCEPTION;

    /**
     * Class type 'java.lang.Error'.
     */
    public static final ClassOrInterfaceType ERROR;

    /**
     * Class type 'java.lang.RuntimeException'.
     */
    public static final ClassOrInterfaceType RUNTIMEEXCEPTION;

    /**
     * Array type '[C'
     */
    public static final ArrayType CHARARRAY;

    /**
     * Initialize all constants.
     */
    static
    {
        VOID = new VoidType();
        NULL = new NullType();
        
        BYTE = new ByteType();
        INTEGER = new IntType();
        LONG = new LongType();
        SHORT = new ShortType();
        CHAR = new CharType();

        DOUBLE = new DoubleType();
        FLOAT = new FloatType();
        
        BOOLEAN = new BooleanType();
        
        OBJECT = ClassOrInterfaceType.get(Constants.JAV_OBJECT);
        CLONEABLE = ClassOrInterfaceType.get(Constants.JAV_CLONEABLE);
        SERIALIZABLE = ClassOrInterfaceType.get(Constants.JAV_SERIALIZABLE);
        CLASS = ClassOrInterfaceType.get(Constants.JAV_CLASS);
        STRING = ClassOrInterfaceType.get(Constants.JAV_STRING);
        THROWABLE = ClassOrInterfaceType.get(Constants.JAV_THROWABLE);
        EXCEPTION = ClassOrInterfaceType.get(Constants.JAV_EXCEPTION);
        ERROR = ClassOrInterfaceType.get(Constants.JAV_ERROR);
        RUNTIMEEXCEPTION =
            ClassOrInterfaceType.get(Constants.JAV_RUNTIME_EXCEPTION);
        
        CHARARRAY = new ArrayType(CHAR, 1);
    }
}
