/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.dis;

import java.text.MessageFormat;

import at.dms.classfile.ClassPath;
import at.dms.compiler.UnpositionedError;

/**
 * This class is the entry point for the KOPI disassembler.
 */
public class Main 
    implements Constants
{
    /**
     * Command-line options.
     */
    protected DisOptions options;

    /**
     * Only main can construct Main.
     */
    private Main(String[] args)
    {
        if (!parseArguments(args)) {
            System.exit(1);
        }

        if (options.classpath == null) {
            String path = System.getProperty("java.class.path");
            ClassPath.init(path == null ? "." : path, true);
        }
        else {
            ClassPath.init(options.classpath, true);
        }

        String[] infiles = options.nonOptions;

        if (infiles.length == 0) {
            options.usage();
            System.err.println(DisMessages.NO_SOURCE_FILE);
            System.exit(1);
        }
        else {
            boolean errorsFound = false;

            for (int i = 0; i < infiles.length; i++) {
                if (options.verbose) {
                    System.err.println(MessageFormat.format
                                       (DisMessages.PROCESSING.getFormat(),
                                        new String[] { infiles[i] }));
                }

                try {
                    disassembleClass(infiles[i]);
                }
                catch (UnpositionedError e) {
                    System.err.println("Error: " + e.getMessage());
                    errorsFound = true;
                }
            }

            System.exit(errorsFound ? 1 : 0);
        }
    }

    /**
     * Reads, and disassembles a class file.
     *
     * @exception UnpositionedError
     * An error occurred.
     */
    private void disassembleClass(String fileName)
        throws UnpositionedError
    {
        Disassembler.disassemble(fileName, options.destination, options);
    }

    /**
     * Entry point to the assembler.
     */
    public static void main(String[] args)
    {
        new Main(args);
    }

    /**
     * Parse command line arguments.
     */
    private boolean parseArguments(String[] args)
    {
        options = new DisOptions();
        return options.parseCommandLine(args);
    }
}
